'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = findTags;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function findTags(filename, source) {
  var ast = babylon.parse(source, {
    sourceType: 'module',
    plugins: ['jsx', 'flow']
  });
  var tags = [];

  function collect(tag, node) {
    (0, _invariant2.default)(tag.loc, 'Missing loc, from node: %s', node.type);
    (0, _invariant2.default)(tag.tagname, 'Missing tagname, from node: %s', node.type);
    tags.push(tag);
  }

  (0, _babelTraverse2.default)(ast, {
    ClassDeclaration: function ClassDeclaration(_ref) {
      var node = _ref.node;

      var tagname = node.id.name;
      collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'c' }, node);
    },
    ClassMethod: function ClassMethod(_ref2) {
      var node = _ref2.node;
      var parentPath = _ref2.parentPath;

      if (node.key.name !== 'constructor') {
        var tagname = node.key.name;
        var options = { class: parentPath.parentPath.node.id.name };
        collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'f', options: options }, node);
      }
    },
    VariableDeclarator: function VariableDeclarator(_ref3) {
      var node = _ref3.node;

      var tagname = node.id.name;
      if (tagname) {
        collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'v' }, node);
      } else {
        // for array desctructuring
        var elements = node.id.elements;
        if (elements && elements.length) {
          for (var i = 0; i < elements.length; i++) {
            var el = elements[i];
            collect({ tagname: el.name, filename: filename, loc: el.loc, type: 'v' }, node);
          }
        }
      }
    },
    ImportDefaultSpecifier: function ImportDefaultSpecifier(_ref4) {
      var node = _ref4.node;

      var tagname = node.local.name;
      collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'i' }, node);
    },
    ImportSpecifier: function ImportSpecifier(_ref5) {
      var node = _ref5.node;

      // id may be null for flow function declarations
      if (node.id) {
        var tagname = node.id.name;
        collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'i' }, node);
      }
    },
    FunctionDeclaration: function FunctionDeclaration(_ref6) {
      var node = _ref6.node;

      // id may be null for flow function declarations
      if (node.id) {
        var tagname = node.id.name;
        collect({ tagname: tagname, filename: filename, loc: node.loc, type: 'f' }, node);
      }
    }
  });

  return tags;
} /**
   * Originally based on code in btags npm package.
   *
   * @copyright 2016 Andrey Popp <8mayday@gmail.com>
   * @copyright 2015 Yahoo Inc. All rights reserved.
   * @license MIT
   */
